
#ifndef LUBISHEV_DB_SQLITE_H
#define LUBISHEV_DB_SQLITE_H

#include <lubishev/DB.h>

#include <sqlite3.h>

namespace lubishev {

class DBSQLite : public DB {
    public:
        DBSQLite();

        class Statement {
            public:
                Statement(sqlite3 *db, const QString &sql = QString());
                ~Statement();

                s32 columnAsS32(s32 columnID);
                s64 columnAsS64(s32 columnID);
                QString columnAsText(s32 columnID);
                // Throws.
                void setSQL(const QString &sql);
                // Returns true if there is anything to read. Throws.
                bool step();

            private:
                void finalize();

                sqlite3 *mDB;
                sqlite3_stmt *mStmt;
                QString mSQL;
        };

        virtual QString createEvent(time_t start, time_t end);
        virtual QStringList eventTags(const QString &id);
        virtual QStringList eventsWithin(time_t start, time_t end);
        virtual void eventTimes(const QString &id, time_t *start, time_t *end);
        virtual void setDir(const QString &dir);
        virtual void setEventTags(const QString &id, const QStringList &list);
        virtual void setEventTimes(const QString &id, time_t start, time_t end);

    private:
        void close();
        // Make sure the necessary tables are present.
        void ensure();
        void open(const QString &fileName);
        s64 tagID(const QString &tag);

        sqlite3 *mDB;
        Statement *mStmt;
};

} // namespace lubishev

#endif // LUBISHEV_DB_SQLITE_H

